package dynaaminen;


/**
 * Esimerkki dynaamisesta taulukosta
 * @author vesal
 * @version 28.2.2023
 * @example
 * <pre name="test">
 *  #THROWS Taulukko.TaulukkoTaysiException
 *  Taulukko luvut = new Taulukko(5);
 *  luvut.add(0); luvut.add(2); luvut.add(99);
 *  luvut.toString() === "0,2,99";
 *  luvut.set(1,4); luvut.toString() === "0,4,99";
 *  int luku = luvut.get(2);
 *  luku === 99;
 *  luvut.get(2) === 99;
 *  luvut.set(21, 4); #THROWS IndexOutOfBoundsException
 *  luvut.add(0); luvut.add(0); //taulukko täyteen
 *  luvut.add(0);  #THROWS Taulukko.TaulukkoTaysiException
 * </pre>
 */
public class Taulukko {
    /** Luokka täyden taulukon poikkeusta varten  */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;
        /**
         * @param viesti viesti joka tulee poikkeuksesta
         */
        public TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    
    private int lkm = 0;
    private int[] alkiot;
    
    
    /**
     * Alutetaan valitun kokoinen taulukko
     * @param koko taulukon koko
     */
    public Taulukko(int koko) {
        alkiot = new int[koko];
    }
    
    
    /**
     * Luodaan oletuskokoinen taulukko
     */
    public Taulukko() {
        this(5);
    }
    
    
    /**
     * Lisätään taulukkoon yksi alkio
     * @param luku lisättävä alkio
     * @throws TaulukkoTaysiException jos tila loppuu
     */
    public void add(int luku) throws TaulukkoTaysiException  {
        if (lkm >= alkiot.length) throw new TaulukkoTaysiException("Tila loppu! Ei saatu lisättyä " + luku);
        alkiot[lkm] = luku;
        lkm++;
    }
    
    
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        String erotin = "";
        for (int i = 0; i < lkm; i++) {
            s.append(erotin);
            s.append(alkiot[i]);
            erotin = ",";
        }
        return s.toString();
    }
    
    
    /**
     * Asetetaan taulukon i's alkio
     * @param i mihin paikkaan astetaan
     * @param luku mikä arvo laitetaan
     */
    public void set(int i, int luku) {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("set i = " + i);
        alkiot[i] = luku;
    }
    
    
    /**
     * Palautetana paikasssa i oleva luku
     * @param i mistä paikasta luku otetaan
     * @return paikassa i olevan luvun arvo
     */
    public int get(int i) {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("get i = " + i);
        return alkiot[i];
    }
    
    /**
     * Kokeillaan taulukkoa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Taulukko luvut = new Taulukko(53);
        try {
            luvut.add(0);
            luvut.add(2);
            luvut.add(99);
            System.out.println(luvut.toString());
            try {
               luvut.get(3);
            } catch (IndexOutOfBoundsException e) {
                // Ei tehdä mitään
            }
            
            luvut.add(199);
            luvut.add(299);
            luvut.add(399);
            luvut.add(499);
            luvut.add(599);
            luvut.add(699);
            luvut.add(799);
        } catch (TaulukkoTaysiException e) {
            // e.printStackTrace();
            System.err.println(e.getMessage());
        } catch (IndexOutOfBoundsException e) {
            // e.printStackTrace();
            System.err.println(e.getMessage());
        }

        
        luvut.set(1,4);
        luvut.set(2,67);
        int a = luvut.get(1);
        System.out.println(a);
        
        System.out.println(luvut.toString());
    }

}
