package dynaaminen;

/**
 * Esimerkki dynaamisesta taulukosta
 * @author vesal
 * @param <TYPE> Tyyppi jota talletetaan
 * @version 28.2.2023
 * @example
 * <pre name="test">
 *  #THROWS Taulukko.TaulukkoTaysiException
 *  Taulukko luvut = new Taulukko(5);
 *  luvut.add(0); luvut.add(2); luvut.add(99);
 *  luvut.toString() === "0,2,99";
 *  luvut.set(1,4); luvut.toString() === "0,4,99";
 *  int luku = luvut.get(2);
 *  luku === 99;
 *  luvut.get(2) === 99;
 *  luvut.set(21, 4); #THROWS IndexOutOfBoundsException
 *  luvut.add(0); luvut.add(0); //taulukko täyteen
 *  luvut.add(0);  #THROWS Taulukko.TaulukkoTaysiException
 * </pre>
 */
public class TaulukkoGen<TYPE> {
    /** Luokka täyden taulukon poikkeusta varten  */
    public static class TaulukkoTaysiException extends Exception {
        private static final long serialVersionUID = 1L;
        /**
         * @param viesti viesti joka tulee poikkeuksesta
         */
        public TaulukkoTaysiException(String viesti) {
            super(viesti);
        }
    }

    
    private int lkm = 0;
    private TYPE[] alkiot;
    
    
    /**
     * Alutetaan valitun kokoinen taulukko
     * @param koko taulukon koko
     */
    @SuppressWarnings("unchecked")
    public TaulukkoGen(int koko) {
        alkiot = (TYPE[])new Object[koko];
    }
    
    
    /**
     * Luodaan oletuskokoinen taulukko
     */
    public TaulukkoGen() {
        this(5);
    }
    
    
    /**
     * Lisätään taulukkoon yksi alkio
     * @param alkio lisättävä alkio
     * @throws TaulukkoTaysiException jos tila loppuu
     */
    public void add(TYPE alkio) throws TaulukkoTaysiException  {
        if (lkm >= alkiot.length) throw new TaulukkoTaysiException("Tila loppu! Ei saatu lisättyä " + alkio);
        alkiot[lkm] = alkio;
        lkm++;
    }
    
    
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder("");
        String erotin = "";
        for (int i = 0; i < lkm; i++) {
            s.append(erotin);
            s.append(alkiot[i]);
            erotin = ",";
        }
        return s.toString();
    }
    
    
    /**
     * Asetetaan taulukon i's alkio
     * @param i mihin paikkaan astetaan
     * @param alkio mikä arvo laitetaan
     */
    public void set(int i, TYPE alkio) {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("set i = " + i);
        alkiot[i] = alkio;
    }
    
    
    /**
     * Palautetana paikasssa i oleva luku
     * @param i mistä paikasta luku otetaan
     * @return paikassa i olevan luvun arvo
     */
    public TYPE get(int i) {
        if ((i < 0) || (lkm <= i)) throw new IndexOutOfBoundsException("get i = " + i);
        return alkiot[i];
    }
    
    /**
     * Kokeillaan taulukkoa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        TaulukkoGen<Integer> luvut = new TaulukkoGen<Integer>();
        try {
            luvut.add(0);
            luvut.add(2);
            luvut.add(99); // Tekee oikeasti luvut.add(new Integer(99));
            System.out.println(luvut);
            luvut.set(1, 4);
            System.out.println(luvut);
            int luku = luvut.get(2); // oik: luku = luvut.get(2).intValue();
            System.out.println("Paikassa 2 on " + luku);
            try {
                luvut.set(21, 4);
            } catch (IndexOutOfBoundsException e) {
                System.out.println("Virhe: " + e.getMessage());
            }
            
            TaulukkoGen<String> jonot = new TaulukkoGen<String>();
            jonot.add("kissa");
            jonot.add("kana");
            jonot.add("koira");
            System.out.println(jonot);   
        } catch (TaulukkoTaysiException e) {
            System.err.println(e.getMessage());
        }
     }

}
