package kerho;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Vois laittaa CRV kortin tekstin
 * @author vesal
 * @version 8.3.2021
 */
public class Harrastukset {
    private Collection<Harrastus> alkiot = new ArrayList<Harrastus>();
    
    /**
     * Alustaminen
     */
    public Harrastukset() {
        // ei tarvii tehdä mitään
    }
    
    
    /**
     * 
     * @param har lisättävä harrastus
     */
    public void lisaa(Harrastus har) {
        alkiot.add(har);
    }
    
    
    /**
     * Haetaan kaikki jäsen harrastukset
     * @param tunnusnro jäsenen tunnusnumero jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Harrastukset harrasteet = new Harrastukset();
     *  Harrastus pitsi21 = new Harrastus(2); harrasteet.lisaa(pitsi21);
     *  Harrastus pitsi11 = new Harrastus(1); harrasteet.lisaa(pitsi11);
     *  Harrastus pitsi22 = new Harrastus(2); harrasteet.lisaa(pitsi22);
     *  Harrastus pitsi12 = new Harrastus(1); harrasteet.lisaa(pitsi12);
     *  Harrastus pitsi23 = new Harrastus(2); harrasteet.lisaa(pitsi23);
     *  Harrastus pitsi51 = new Harrastus(5); harrasteet.lisaa(pitsi51);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = harrasteet.annaHarrastukset(3);
     *  loytyneet.size() === 0; 
     *  loytyneet = harrasteet.annaHarrastukset(1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = harrasteet.annaHarrastukset(5);
     *  loytyneet.size() === 1; 
     *  loytyneet.get(0) == pitsi51 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(int tunnusnro) {
        List<Harrastus> loydetyt = new ArrayList<Harrastus>();
        for (Harrastus har: alkiot) // iteraattori
            if (har.getJasenNro() == tunnusnro) loydetyt.add(har);
        return loydetyt;
    }
    
    /**
     * Testiohjelma harrastuksille
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Harrastukset harrasteet = new Harrastukset();
        Harrastus pitsi1 = new Harrastus();
        pitsi1.vastaaPitsinNyplays(2);
        Harrastus pitsi2 = new Harrastus();
        pitsi2.vastaaPitsinNyplays(1);
        Harrastus pitsi3 = new Harrastus();
        pitsi3.vastaaPitsinNyplays(2);
        Harrastus pitsi4 = new Harrastus();
        pitsi4.vastaaPitsinNyplays(2);
    
        harrasteet.lisaa(pitsi1);
        harrasteet.lisaa(pitsi2);
        harrasteet.lisaa(pitsi3);
        harrasteet.lisaa(pitsi4);
    
        System.out.println("============= Harrastukset testi =================");
    
        var harrastukset2 = harrasteet.annaHarrastukset(1);
    
        for (Harrastus har : harrastukset2) {
            System.out.print(har.getJasenNro() + " ");
            har.tulosta(System.out);
        }
    }
}
