package fxKerho;

import static fxKerho.TietueDialogController.getFieldId; 
import java.awt.Desktop;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.StringGrid;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.GridPane;
import kerho.Harrastus;
import kerho.Jasen;
import kerho.Kerho;
import kerho.SailoException;

/**
 * Luokka kerhon käyttöliittymän tapahtumien hoitamiseksi.
 * 
 * @author vesal
 * @version 31.12.2015
 * @version 5.1.2015
 * @version 5.2.2017 - lisätty jäsenien käsittely 
 * @version 5.2.2017 - lisätty harrastusten käsittely 
 */
public class KerhoGUIController implements Initializable {

    @FXML private TextField editNimi;
    @FXML private TextField editHetu;
    @FXML private TextField editKatuosoite;
    @FXML private TextField editPostinumero;
    @FXML private GridPane gridJasen;

    @FXML private TextField hakuehto;
    @FXML private ComboBoxChooser<String> cbKentat;
    @FXML private Label labelVirhe;
    @FXML private ScrollPane panelJasen;
    @FXML private ListChooser<Jasen> chooserJasenet;
    @FXML private StringGrid<Harrastus> tableHarrastukset;

    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();      
    }

    
    @FXML private void handleHakuehto() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        if ( jasenKohdalla != null )
            hae(jasenKohdalla.getTunnusNro()); 
    }
    
    
    @FXML private void handleTallenna() {
        tallenna();
    }
    
    
    @FXML private void handleAvaa() {
        avaa();
    }
    
    
    @FXML private void handleTulosta() {
        TulostusController tulostusCtrl = TulostusController.tulosta(null); 
        tulostaValitut(tulostusCtrl.getTextArea()); 
    } 

    
    @FXML private void handleLopeta() {
        tallenna();
        Platform.exit();
    } 

    
    @FXML private void handleUusiJasen() {
        uusiJasen();
    }
    
    
    @FXML private void handleMuokkaaJasen() {
        muokkaa(kentta);
    }
    
    
    @FXML private void handlePoistaJasen() {
        Dialogs.showMessageDialog("Vielä ei osata poistaa jäsentä");
    }
    
     
    @FXML private void handleUusiHarrastus() {
        uusiHarrastus(); 
    }
    

    @FXML private void handleMuokkaaHarrastus() {
        muokkaaHarrastusta();
    }
    

    @FXML private void handlePoistaHarrastus() {
        Dialogs.showMessageDialog("Ei osata vielä poistaa harrastusta");
    }
    

    @FXML private void handleApua() {
        avustus();
    }
    

    @FXML private void handleTietoja() {
        // Dialogs.showMessageDialog("Ei osata vielä tietoja");
        ModalController.showModal(KerhoGUIController.class.getResource("AboutView.fxml"), "Kerho", null, "");
    }
    

//===========================================================================================    
// Tästä eteenpäin ei käyttöliittymään suoraan liittyvää koodia    
    
    private String kerhonnimi = "kelmit";
    private Kerho kerho;
    private TextField[] edits;
    private int kentta = 0; 
    private static Harrastus apuharrastus = new Harrastus();
    
    
    /**
     * Tekee tarvittavat muut alustukset, nyt vaihdetaan GridPanen tilalle
     * yksi iso tekstikenttä, johon voidaan tulostaa jäsenten tiedot.
     * Alustetaan myös jäsenlistan kuuntelija 
     */
    protected void alusta() {
        // panelJasen.setContent(areaJasen);
        panelJasen.setFitToHeight(true);
        
        chooserJasenet.clear();
        chooserJasenet.addSelectionListener(e -> naytaJasen());
        
        edits = TietueDialogController.luoKentat(gridJasen, new Jasen());      
        for (TextField edit: edits)   
            if ( edit != null ) {   
                edit.setEditable(false);   
                edit.setOnMouseClicked(e -> { if ( e.getClickCount() > 1 ) muokkaa(getFieldId(edit,kentta)); });  
                edit.focusedProperty().addListener((a,o,n) -> kentta = getFieldId(edit,kentta)); 
            } 
        
        // alustetaan harrastustaulukon otsikot 
        int eka = apuharrastus.ekaKentta(); 
        int lkm = apuharrastus.getKenttia(); 
        String[] headings = new String[lkm-eka]; 
        for (int i=0, k=eka; k<lkm; i++, k++) headings[i] = apuharrastus.getKysymys(k); 
        tableHarrastukset.initTable(headings); 
        tableHarrastukset.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY); 
        tableHarrastukset.setEditable(false); 
        tableHarrastukset.setPlaceholder(new Label("Ei vielä harrastuksia")); 
         
        // Tämä on vielä huono, ei automaattisesti muutu jos kenttiä muutetaan. 
        tableHarrastukset.setColumnSortOrderNumber(1); 
        tableHarrastukset.setColumnSortOrderNumber(2); 
        tableHarrastukset.setColumnWidth(1, 60); 
        tableHarrastukset.setColumnWidth(2, 60); 
        
        tableHarrastukset.setEditable(false);
        tableHarrastukset.setOnMouseClicked( e -> { if ( e.getClickCount() > 1 ) muokkaaHarrastusta(); } );
        tableHarrastukset.setOnKeyPressed( e -> {if ( e.getCode() == KeyCode.F2 ) muokkaaHarrastusta();}); 
    }

    
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    private void setTitle(String title) {
        ModalController.getStage(hakuehto).setTitle(title);
    }
    
    
    /**
     * Alustaa kerhon lukemalla sen valitun nimisestä tiedostosta
     * @param nimi tiedosto josta kerhon tiedot luetaan
     * @return null jos onnistuu, muuten virhe tekstinä
     */
    protected String lueTiedosto(String nimi) {
        kerhonnimi = nimi;
        setTitle("Kerho - " + kerhonnimi);
        try {
            kerho.lueTiedostosta(nimi);
            hae(0);
            return null;
        } catch (SailoException e) {
            hae(0);
            String virhe = e.getMessage(); 
            if ( virhe != null ) Dialogs.showMessageDialog(virhe);
            return virhe;
        }
     }

    
    /**
     * Kysytään tiedoston nimi ja luetaan se
     * @return true jos onnistui, false jos ei
     */
    public boolean avaa() {
        String uusinimi = KerhonNimiController.kysyNimi(null, kerhonnimi);
        if (uusinimi == null) return false;
        lueTiedosto(uusinimi);
        return true;
    }

    
    /**
     * Tietojen tallennus
     * @return null jos onnistuu, muuten virhe tekstinä
     */
    private String tallenna() {
        try {
            kerho.tallenna();
            return null;
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Tallennuksessa ongelmia! " + ex.getMessage());
            return ex.getMessage();
        }
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true jos saa sulkea sovelluksen, false jos ei
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }
    
    
    /**
     * Näyttää listasta valitun jäsenen tiedot, tilapäisesti yhteen isoon edit-kenttään
     */
    protected void naytaJasen() {
        Jasen jasen = chooserJasenet.getSelectedObject();
        if (jasen == null) return;

        TietueDialogController.naytaTietue(edits, jasen); 
        naytaHarrastukset(jasen);
    }

    
    private void naytaHarrastukset(Jasen jasen) {
        tableHarrastukset.clear();
        if (jasen == null) return;
        List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
        if (harrastukset.size() == 0) return;
        for (Harrastus har: harrastukset) 
            naytaHarrastus(har);
    }
  
    
    private void naytaHarrastus(Harrastus har) {
        int kenttia = har.getKenttia(); 
        String[] rivi = new String[kenttia-har.ekaKentta()]; 
        for (int i=0, k=har.ekaKentta(); k < kenttia; i++, k++) 
            rivi[i] = har.anna(k); 
        tableHarrastukset.add(har,rivi);
    }
    
    
    private void muokkaaHarrastusta() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        int r = tableHarrastukset.getRowNr();
        if ( r < 0 ) return; // klikattu ehkä otsikkoriviä
        Harrastus har = tableHarrastukset.getObject();
        if ( har == null ) return;
        int k = tableHarrastukset.getColumnNr()+har.ekaKentta();
        try {
            har = TietueDialogController.kysyTietue(null, har.clone(), k);
            if ( har == null ) return;
            kerho.korvaaTaiLisaa(har); 
            naytaHarrastukset(jasenKohdalla); 
            tableHarrastukset.selectRow(r);  // järjestetään sama rivi takaisin valituksi
        } catch (CloneNotSupportedException  e) { /* clone on tehty */  
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia lisäämisessä: " + e.getMessage());
        }
    }
    

    private void muokkaa(int k) {
        Jasen jasen = chooserJasenet.getSelectedObject(); 
        if (jasen == null) return;
        try {
            jasen = jasen.clone();
        } catch (CloneNotSupportedException e) {
            // Ei voi tapahtua
        }
        jasen = TietueDialogController.kysyTietue(null, jasen, k);
        if (jasen == null) return;
        try {
            kerho.korvaaTaiLisaa(jasen);
        } catch (SailoException e) {
            // TODO: näytä dialogi virheestä
        }
        hae(jasen.getTunnusNro());
    }
    
    
    /**
     * Hakee jäsenten tiedot listaan
     * @param jnro jäsenen numero, joka aktivoidaan haun jälkeen
     */
    protected void hae(int jnro) {
        int k = cbKentat.getSelectionModel().getSelectedIndex();
        String ehto = hakuehto.getText(); 
        if (k > 0 || ehto.length() > 0)
            naytaVirhe(String.format("Ei osata hakea (kenttä: %d, ehto: %s)", k, ehto));
        else
            naytaVirhe(null);
        
        chooserJasenet.clear();

        int index = 0;
        Collection<Jasen> jasenet;
        try {
            jasenet = kerho.etsi(ehto, k);
            int i = 0;
            for (Jasen jasen:jasenet) {
                if (jasen.getTunnusNro() == jnro) index = i;
                chooserJasenet.add(jasen.getNimi(), jasen);
                i++;
            }
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Jäsenen hakemisessa ongelmia! " + ex.getMessage());
        }
        chooserJasenet.setSelectedIndex(index); // tästä tulee muutosviesti joka näyttää jäsenen
    }


    /**
     * Luo uuden jäsenen jota aletaan editoimaan 
     */
    protected void uusiJasen() {
        Jasen uusi = new Jasen();
        uusi = TietueDialogController.kysyTietue(null, uusi, uusi.ekaKentta()); 
        if (uusi == null) return;
        uusi.rekisteroi();
        try {
            kerho.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
        hae(uusi.getTunnusNro());
    }
    

    /**
     * Tekee uuden tyhjän harrastuksen editointia varten
     */
    private void uusiHarrastus() {
        Jasen jasenKohdalla = chooserJasenet.getSelectedObject(); 
        if ( jasenKohdalla == null ) return;

        Harrastus uusi = new Harrastus(jasenKohdalla.getTunnusNro());
        uusi = TietueDialogController.kysyTietue(null, uusi, 0);
        if ( uusi == null ) return;
        uusi.rekisteroi();
        try {
            kerho.lisaa(uusi);
        } catch (SailoException e) {
            // Näytä dialogi
        }
        naytaHarrastukset(jasenKohdalla); 
        tableHarrastukset.selectRow(1000);  // järjestetään viimeinen rivi valituksi
    }


    /**
     * @param kerho Kerho jota käytetään tässä käyttöliittymässä
     */
    public void setKerho(Kerho kerho) {
        this.kerho = kerho;
        naytaJasen();
    }

    
    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/2019k/ht/vesal");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    }

    
    /**
     * Tulostaa jäsenen tiedot
     * @param os tietovirta johon tulostetaan
     * @param jasen tulostettava jäsen
     */
    public void tulosta(PrintStream os, final Jasen jasen) {
        os.println("----------------------------------------------");
        jasen.tulosta(os);
        os.println("----------------------------------------------");
        List<Harrastus> harrastukset = kerho.annaHarrastukset(jasen);
        for (Harrastus har:harrastukset) 
            har.tulosta(os);     
    }
    
    
    /**
     * Tulostaa listassa olevat jäsenet tekstialueeseen
     * @param text alue johon tulostetaan
     */
    public void tulostaValitut(TextArea text) {
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(text)) {
            os.println("Tulostetaan kaikki jäsenet");
            Collection<Jasen> jasenet = kerho.etsi("", -1); 
            for (Jasen jasen:jasenet) { 
                tulosta(os, jasen);
                os.println("\n\n");
            }
        } catch (SailoException ex) { 
            Dialogs.showMessageDialog("Jäsenen hakemisessa ongelmia! " + ex.getMessage()); 
        }
    }
}
