package stringgrid;

import java.util.Collection;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.StringGrid;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import kanta.SailoException;
import kerho.Jasen;
import kerho.Kerho;

/**
 * Esimerkki Jäsenen käyttämisestä StringGridissä
 * @author vesal
 * @version 3.4.2016
 */
public class JasenStringGrid extends Application {
    @Override
    public void start(Stage primaryStage) {
        try {
            final BorderPane root = new BorderPane();
            StringGrid<Jasen> grid = new StringGrid<>();
            root.setCenter(grid);
            
            primaryStage.setScene(new Scene(root));
            primaryStage.setTitle("Kerho");
            
            Kerho kerho = new Kerho();
            kerho.lueTiedostosta("kelmit");
            alusta(kerho, kerho.etsi("*", 0),grid);
            
            primaryStage.setOnCloseRequest((event) -> {
                try {
                    kerho.tallenna();
                } catch (SailoException e) {
                    // TODO viesti
                }
            } );

            primaryStage.show();
        } catch(Exception e) {
            e.printStackTrace();
        }   
    }

    
    private void alusta(Kerho kerho, Collection<Jasen> jasenet, StringGrid<Jasen> grid) {
        Jasen apujasen = new Jasen();
        int eka = apujasen.ekaKentta();
        int lkm = apujasen.getKenttia();
        String[] headings = new String[lkm-eka];
        for (int k=eka; k<lkm; k++) headings[k-eka] = apujasen.getKysymys(k);
        grid.initTable(headings);

        for (int k=eka; k<lkm; k++) grid.setAlignment(k-eka, apujasen.getSijainti(k));
        
        grid.setOnCellString( (g, jasen, defValue, r, c) -> jasen.anna(c+eka) );
        grid.setOnCellValue( (g, jasen, defValue, r, c) -> jasen.getAvain(c+eka) );

        grid.setTableMenuButtonVisible(true); // menu, josta voi valita sarakkeet
        
        grid.setEditable(true);
        grid.setOnGridLiveEdit((g, jasen, defValue, r, c, edit) -> {
            String virhe = jasen.aseta(c+eka,defValue);
            if ( virhe == null ) {
                try {
                    kerho.korvaaTaiLisaa(jasen); // jotta saadaan muutos
                } catch (kanta.SailoException e) {
                    //
                } 
                edit.setStyle(null);
                Dialogs.setToolTipText(edit,"");
            } else {
                edit.setStyle("-fx-background-color: red");
                Dialogs.setToolTipText(edit,virhe);
            }
            return defValue;
        });
        
        grid.add(jasenet);
    }


    /**
     * Käynnistetään käyttöliittymä 
     * @param args komentorivin parametrit
     */
    public static void main(String[] args) {
        launch(args);
    }
}