package olio;

/**
 * Eka esimerkki olioista
 * @author vesal
 * @version 23.1.2012
 */
public class Aika {

    private int h; // attribuutti
    private int m;


    /**
     * Alustetaan olio 0:00
     */
    public Aika() { // oletusmuodostaja
        aseta(0, 0);
    }


    /**
     * Alustetaan olio
     * @param h olion tunnit
     * @param m olion minuutit
     */
    public Aika(int h, int m) { // muodostaja
        aseta(h, m);
    }


    /**
     * Asettaa uuden ajan ja pitää huolen että aika on aina oikeaa muotoa.
     * @param ih asetettava tuntimäärä
     * @param im asetettava minuuttimäärä
     * @return montako vuorokautta jäi yli
     * <pre name="test">
     *  Aika a1 = new Aika();
     *  a1.aseta(12,15); a1.toString() === "12:15";
     *  a1.aseta(15,45); a1.toString() === "15:45";
     *  a1.aseta(-49,-125); a1.toString() === "20:55";
     * </pre>
     */
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit väille 0-59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit välille 0-23

        if (tm < 0) { tm += 60; th--;  } //negatiiviset arvot
        if (th < 0) { th += 24; vrk--; }

        this.h = th; // asetetaan lasketut arvot attribuutteeihin
        this.m = tm;
        return vrk; // motako vuorokautta jäi yli
    }


    /**
     * @return aika muodossa 09:05
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,15);
     *   a.toString() === "12:15";
     *   a = new Aika(9,5);
     *   a.toString() === "09:05";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", this.h, this.m);
    }


    /**
     * Lisää aikaan lisamin
     * @param lisamin lisättävät minuutit
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,15);
     *   a.lisaa(50);
     *   a.toString() === "13:05";
     * </pre>
     */
    public void lisaa(int lisamin) {
        //int minyht = 60*h + m + lisamin;
        //h = minyht / 60;
        //m = minyht % 60;
        aseta(h, m + lisamin);
    }


    /**
     * @return nykyiset tunnit
     */
    public int getH() {
        return h;
    }


    /**
     * @param ih asetettavat tunnit
     */
    public void setH(int ih) {
        aseta(ih, this.m);
    }


    /**
     * @return nykyiset tunnit
     */
    public int getM() {
        return m;
    }


    /**
     * @param im asetettavat minuutit
     */
    public void setM(int im) {
        aseta(this.h, im);
    }


    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Aika a1 = new Aika(12, 15); // olio
        Aika a2 = new Aika(13, 55);
        // String s = a1.toString();
        System.out.println(a1);
        System.out.println(a2);
        a1.lisaa(50);
        System.out.println(a1);
        {
            Aika a3 = new Aika(13, 55);
            a3.lisaa(51);
        }
        a1.m = 97;
    }

}