package valinta;

/**
 * Luokka äänestyksen yhdelle vaihtoehdolle
 * @author Vesa Lappalainen
 * @version 1.0, 27.02.2003
 */
public class Valinta {

    private final char kirjain;
    private final String teksti;
    private int maara;


    /**
     * Alustetaan äänestyskohde valituilla arvoilla
     * @param kirjain kirjain jolla äänestetään
     * @param teksti mitä näytetään äänestyskohteeksi
     */
    public Valinta(char kirjain, String teksti) {
        this.kirjain = kirjain;
        this.teksti = teksti;
    }


    /**
     * Kohteen tulostettava teksti
     * @return esim. K = Kyllä
     *  Valinta val = new Valinta('K',"Kyllä"); 
     *  val.getTulostus() === "K = Kyllä"; 
     */
    public String getTulostus() {
        return kirjain + " = " + teksti;
    }


    /**
     * Montako ääntä kohde on saanut
     * @return äänien määrä
     */
    public int getMaara() {
        return maara;
    }


    /**
     * Lisää yksi ääni kohteelle
     * @example
     * <pre name="test">
     *  Valinta val = new Valinta('K',"Kyllä");
     *  val.getMaara() === 0;
     *  val.lisaa();
     *  val.getMaara() === 1;
     *  val.lisaa();
     *  val.getMaara() === 2;
     * </pre>
     */
    public void lisaa() {
        maara++;
    }


    /**
     * Kohteen äänestyskirjain
     * @return kohteen kirjain
     */
    public char getKirjain() {
        return kirjain;
    }


    /**
     * Kohteen kuvaileva teksti
     * @return kohteen teksti
     */
    public String getTeksti() {
        return teksti;
    }


    /**
     * Testataan äänestyskohdetta
     * @param args Ei käytössä
     */
    public static void main(String[] args) {
        Valinta val = new Valinta('K', "Kyllä");
        String tulostus = val.getTulostus();
        System.out.println(tulostus);
        int lkm = val.getMaara();
        System.out.println(lkm);
        val.lisaa();
        lkm = val.getMaara();
        System.out.println(lkm);
    }
}